package nio;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.nio.MappedByteBuffer;
import java.nio.channels.FileChannel;
import java.nio.charset.Charset;
import java.nio.charset.CharsetDecoder;
import java.nio.charset.CharsetEncoder;
import java.nio.charset.CoderResult;

public class FichierIO {
    private String encodage;

    public static void main(String[] args) {
        FichierIO io = new FichierIO("UTF-16");
        File fichier = new File("document.txt");
        io.ecrire("Un contenu quelconque...", fichier, false);
        System.out.println("Contenu :\n" + io.lire(fichier));
    }

    public FichierIO(String encodage) {
        this.encodage = encodage;
    }

    public void ecrire(String contenu, File fichier, boolean ajout) {
        try {
            FileOutputStream fos = new FileOutputStream(fichier, ajout);
            FileChannel fc = fos.getChannel();

            Charset latin = Charset.forName(this.encodage);
            CharsetEncoder encodeur = latin.newEncoder();

            if (contenu == null)
                return;
            CharBuffer cb = CharBuffer.wrap(contenu);
            int taille = (int)(cb.length() * encodeur.maxBytesPerChar());
            ByteBuffer bb = ByteBuffer.allocate(taille);
            System.out.println(this.getEtatInterne(encodeur));
            CoderResult code = encodeur.encode(cb, bb, false);
            if (code.isMalformed() || code.isUnmappable())
                System.out.println("Une erreur d'criture s'est produite !");
            cb.flip();
            bb.flip();
            System.out.println("Taille Caractres / Octets / Maximum : " 
                                + contenu.length() + " / " 
                                + bb.limit() + " / " 
                                + taille);
            System.out.println("Ecriture du fichier " + fichier.toString());
            fc.write(bb);

            fc.close();
        }
        catch (java.io.FileNotFoundException e) {
            e.printStackTrace();
        }
        catch (IOException e) {
            e.printStackTrace();
        }
    }

    public String lire(File fichier) {
        try {
            if (fichier.exists()) {
                FileInputStream fis = new FileInputStream(fichier);
                FileChannel fc = fis.getChannel();
                int taille = (int) fc.size();
                MappedByteBuffer mpb = 
                                fc.map(FileChannel.MapMode.READ_ONLY, 0, taille);

                Charset latin = Charset.forName(this.encodage);
                CharsetDecoder decodeur = latin.newDecoder();
                CharBuffer cb = CharBuffer.allocate(taille);
                System.out.println(this.getEtatInterne(decodeur));
                CoderResult code = decodeur.decode(mpb, cb, false);
                if (code.isMalformed() || code.isUnmappable())
                    System.out.println("Une erreur de lecture s'est produite !");
                mpb.flip();
                cb.flip();
                fc.close();
                System.out.println("Lecture du fichier " + fichier.toString());
                return cb.toString();
            }
        }
        catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }
    
    private String getEtatInterne(Object codeur) {
        StringBuffer res = new StringBuffer("Jeu de caractres : ");
        if(codeur instanceof CharsetDecoder) {
            CharsetDecoder cd = (CharsetDecoder)codeur;
            res.append(cd.charset().toString());
            res.append("\nMoyenne de caractres par octet : ");
            res.append(cd.averageCharsPerByte());
            res.append("\nNombre de caractres maximum par octet : ");
            res.append(cd.maxCharsPerByte());
        }
        else if(codeur instanceof CharsetEncoder) {
            CharsetEncoder ce = (CharsetEncoder)codeur;
            res.append(ce.charset().toString());
            res.append("\nMoyenne d'octets par caractre : ");
            res.append(ce.averageBytesPerChar());
            res.append("\nNombre d'octets maximum par caractre : ");
            res.append(ce.maxBytesPerChar());
        }
        return res.toString();
    }
}
